# Spec file for xfs linux-2.6 kernel modules

# rpm is built like this:
# rpmbuild --rebuild --target i686 --define "kernel_topdir /lib/modules/2.6.9-42.EL/build" kernel-module-xfs-2.6.9-42.EL-0.2-1.src.rpm

%define kmod_name xfs

# if kernel_topdir isn't defined on rpm build line, build against current kernel
#%%define _kernel_topdir %{?kernel_topdir:%{kernel_topdir}}%{!?kernel_topdir:%(echo /lib/modules/`uname -r`/build)}

%{!?kernel_topdir: %define kernel_topdir %(echo /lib/modules/`uname -r`/build)}

# Work out kernel uname, version, release, type, etc for provides/requires

%define kernel_uname %(grep -s UTS_RELEASE %{kernel_topdir}/include/linux/version.h | awk '{print $3}' | sed s/\\"//g)

# kernel_type is everything else at the end of the uname
%define kernel_type %(echo %{kernel_uname} | grep -o "deault$\\|smp$\\|bigsmp$\\|bigmem$\\|debug$\\|hugemem$\\|largesmp$\\|sn2$")
# kernel_type, IF it exists, has leading "-" in the kernel rpm name
%define rpm_kernel_type %(test ! -z "%{kernel_type}" && echo -%{kernel_type})

%define kernel_ver %(echo %{kernel_uname} | awk -F - '{print $1}')
%define kernel_rel %(echo %{kernel_uname} | awk -F - '{print $2}' | sed -e "s/%{kernel_type}$//")

Summary		: XFS filesystem kernel modules
Name		: kernel-module-%{kmod_name}-%{kernel_uname}
Version		: 0.2
Release		: 1
Packager	: Eric Sandeen <sandeen-centos@sandeen.net>
License		: GPL
Group		: System Environment/Kernel
URL		: http://oss.sgi.com/projects/xfs
BuildRoot	: %{_tmppath}/%{name}-%{version}-%{release}-buildroot
Provides	: kernel-module
Provides	: kernel-modules = %{kernel_uname}
Provides	: %{kmod_name}-kmod = %{?epoch:%{epoch}:}%{version}-%{release}
Provides	: kernel-module-%{kmod_name} = %{version}-%{release}
Requires	: kernel%{rpm_kernel_type} >= 2.6.9-42.EL
Requires	: kernel%{rpm_kernel_type} = %{kernel_ver}-%{kernel_rel}
Requires	: /sbin/depmod
Requires	: /sbin/mkinitrd

# different name here
Source0		: %{kmod_name}-kern.tar.gz
Source1		: config.xfs

# Allow the code to build as an out-of-tree module
Patch0		: %{kmod_name}_ootbuild.patch

# Update patches from the sles9 kernel since first release
Patch1		: xfs-kern-202363a-fix-xfs_finish_reclaim_all-umount-deadlock
Patch2		: xfs-log-diag
Patch3		: xfs-kern-25238a-quota-trans-diag
Patch4		: xfs-ftruncate-stale-data
Patch5		: xfs-kern-930841-default_acl_enospc_fix
Patch6		: xfs-kern-25687a-sles9sp3-iunpin-reclaim-fix
Patch7		: xfs-kern-26040a-do-not-dirty-inode-being-freed
Patch8		: xfs-kern-26347a-fix-race-on-link

# This patchset is for making the xfs codebase in sles9 work
# with an RHEL4 kernel.
Patch100	: xfs-version.patch
Patch101	: xfs_qsort.patch
Patch102	: xfs_proc_args.patch
Patch103	: xfs_export_ops.patch
Patch104	: xfs_write_inode_return.patch
Patch105	: xfs_dmapi_address_space.patch
# Needed for kernels < 2.6.9-42.EL, uncomment below
Patch106	: xfs_direct_io_locking.patch

%description
The XFS(tm) Filesystem from Silicon Graphics, Inc.
XFS is a highly scalable, high-performance journaling filesystem that provides 
rapid recovery from system crashes.

%prep

%setup -n %{kmod_name}
%patch0 -p1

%patch1 -p3
%patch2 -p3
%patch3 -p3
%patch4 -p3
%patch5 -p3
%patch6 -p3
%patch7 -p3
%patch8 -p3

%patch100 -p1
%patch101 -p1
%patch102 -p1
%patch103 -p1
%patch104 -p1
%patch105 -p1
#%%patch106 -p1

echo "#define XFS_VERSION_STRING \"SGI XFS from %{name}-%{version}-%{release} rpm\"" > linux-2.6/xfs_module_version.h 

cp %{SOURCE1} .

%build
make TOPDIR=%{kernel_topdir} modules

%install
make TOPDIR=%{kernel_topdir} INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install
# Temporarily executable for stripping, fixed later in %%files.
chmod u+x $RPM_BUILD_ROOT/lib/modules/%{kernel_uname}/extra/*

%clean
[ "$RPM_BUILD_ROOT" != "/" ] && rm -rf $RPM_BUILD_ROOT ]

%files
%defattr(644,root,root,755)
/lib/modules/%{kernel_uname}/extra

# Note, this mkinitrd command is unique to RH I believe

%post
if [ -e /boot/System.map-%{kernel_uname} ]; then
	/sbin/depmod -ae -F /boot/System.map-%{kernel_uname} %{kernel_uname}
fi

if (stat -f / | grep -q 0x58465342) || (stat -f / | grep -qi XFS); then
  echo "It appears that your root filesytem is on XFS."
  echo "Creating a new initrd /boot/initrd-%{kernel_uname}.img"
  echo "for kernel %{kernel_uname} with command:"
  echo "  /sbin/mkinitrd -f /boot/initrd-%{kernel_uname}.img %{kernel_uname}"
  echo "If you use lilo, you should re-run the lilo command."

  /sbin/mkinitrd -f /boot/initrd-%{kernel_uname}.img %{kernel_uname}
fi

echo "###############################################################"
echo "#                                                             #"
echo "#                        NOTICE                               #"
echo "# This xfs module rpm is reasonably tested, but problems may  #"
echo "# still exist.  In particular, you may have issues with the   #"
echo "# 4KSTACKS option on CentOS-4 ia32 kernels, depending on your #"
echo "# IO hardware, layering, nfs usage, etc.                      #"
echo "#                                                             #"
echo "# You may report issues to the linux-xfs@oss.sgi.com list,    #"
echo "# to the packager, Eric Sandeen <sandeen-centos@sandeen.net>, #"
echo "# or to the CentOS bugs database at http://bugs.centos.org/   #"
echo "#                                                             #"
echo "#            (Please also report successes!)                  #"
echo "#                                                             #"
echo "###############################################################"


# Could do an "are you sure? y/n" thing but that might get annoying.
%preun
if (stat -f / | grep -q 0x58465342) || (stat -f / | grep -qi XFS); then
  echo "It appears that your root filesystem is on XFS."
  echo "Note that kernel %{kernel_uname} might not"
  echo "boot without this package."
fi

%postun
if [ -e /boot/System.map-%{kernel_uname} ]; then
       /sbin/depmod -ae -F /boot/System.map-%{kernel_uname} %{kernel_uname}
fi

%changelog 
* Mon Nov 20 2006 - Johnny Hughes <johnny@centos.org>
- added largesmp kernel support for x86_64
- removed vendor tag as all packages are released with CentOS as the vendor.
- Modified the warning to be CentOS (instead of Red Hat) specific.
- Added a help at the top of the spec showing how to pass in defines

* Tue Nov 13 2006 - sandeen-centos@sandeen.net
- removed xfs_direct_io_locking.patch, RHEL4U4 has the fix now.
- Update to xfs codebase from SLES9_SP3_BRANCH_20061107171129
- xfs-kern-26347a-fix-race-on-link: fix race on link
  (191713, SGI:PV953287).
- xfs-kern-26040a-do-not-dirty-inode-being-freed:
  Don't dirty the inode if it being freed in xfs_iunpin (179117,
  SGI:PV952967).
- xfs-kern-25687a-sles9sp3-iunpin-reclaim-fix: Fix
  an inode use-after-free during an unpin (SGI:PV946321, 142533).
- xfs-kern-930841-default_acl_enospc_fix:
  Default acl ENOSPC fix (133990, SGI:PV930841).
- xfs-kern-25238a-quota-trans-diag: quota trans diag
  (131262, SGI:PV931456).
- xfs-ftruncate-stale-data: XFS ftruncate() bug
  could expose stale data (151055).
- xfs-log-diag: log_runout_diagnostics (131262,
  SGI:PV947110).
- xfs-kern-202363a-fix-xfs_finish_reclaim_all-umount-deadlock:
  fix xfs_finish_reclaim_all umount deadlock (132358,
  SGI:PV943821).
- clean up specfile:
  default to building against running kernel
  try to match fedora conventions slightly better
  re-enable debuginfo pkg building, module stripping
  fix recognition of root on xfs, (grep -i XFS)
  remove unused -source subpackage
